/****************************************************************************
**	File:	FILEUSE.C
**
**	Desc:	Sample code to show how to determine which connections are using a
**			file; and how to obtain lock info.
**
**	Disclaimer:
**
**		Novell, Inc. makes no representations or warranties with respect to
**		any NetWare software, and specifically disclaims any express or
**		implied warranties of merchantability, title, or fitness for a
**		particular purpose.  
**
**		Distribution of any NetWare software is forbidden without the
**		express written consent of Novell, Inc.  Further, Novell reserves
**		the right to discontinue distribution of any NetWare software.
**
**		Novell is not responsible for lost profits or revenue, loss of use
**		of the software, loss of data, costs of re-creating lost data, the
**		cost of any substitute equipment or program, or claims by any party
**		other than you.  Novell strongly recommends a backup be made before
**		any software is installed.   Developer support for this software
**		may be provided at the discretion of Novell.
**
**	QMK386 options used:
**
**		None
**
**	Programmers:
**
**		Ini	Who						Firm
**		-----------------------------------------------------------------------
**		ABJ	Adam B. Jerome			Novell Technical Support
**
**	History:
**
**		When		Who	What
**		-----------------------------------------------------------------------
**		01-25-95	ABJ	First code.
*/

/****************************************************************************
**	Compiler setup.
*/
	/*------------------------------------------------------------------------
	**	ANSI
	*/
	#include <stdlib.h>
	#include <stdio.h>
	#include <errno.h>
	#include <string.h>

	/*------------------------------------------------------------------------
	**	NetWare
	*/
	#include <niterror.h>
	#include <nwenvrn1.h>
	#include <nwmisc.h>
	#include <nwdir.h>
	#include <fileengd.h>
	#include <nwconn.h>

/****************************************************************************
**	Program start.
*/
void main(int argC, char *argV[])
	{
	CONN_USING_FILE_REQUEST 	 conReq;
	CONN_USING_FILE_REPLY		*conRep = NULL;
	CONN_USING_FILE_RECORD_386	*conRepRec;

	char objectName[47+1];
	WORD objectType;
	long objectID;
	BYTE loginTime[7];

	int	rVal;
	char *fullPath = "";
	LONG	volumeNumber;
	LONG	NSDirBase;
	LONG	DOSDirBase;
	int	nCnt;

	/*------------------------------------------------------------------------
	**	Parse command line args.
	*/
	if(argC != 2)
		{
		fprintf(stderr, "ERROR: Invalid number of command line args.\n");
		goto END_ERR;
		}
	fullPath=argV[1];

	/*------------------------------------------------------------------------
	**	Parse the fullPath into a volume and directoryID.
	**
	**	NOTE: directoryID, dirBase and directoryNumber are different terms for
	**			the same thing.
	*/
	rVal=NWGetDirBaseFromPath(
		/*	I-	path				*/	fullPath,
		/*	I-	nameSpace		*/	0,					/* 0 = DOS */
		/*	-O	volNum			*/	&volumeNumber,
		/*	-O	NSDirBase		*/	&NSDirBase,
		/*	-O	NSDOSDirBase	*/	&DOSDirBase
		);
	if(rVal != 0)
		{
		switch(rVal)
			{
			case ERR_INVALID_PATH:
				fprintf(stderr, "ERROR: NWGetDirBaseFromPath() indicates INVALID PATH.\n");
				break;

			default:
				fprintf(stderr, "ERROR: NWGetDirBaseFromPath() indicates %d, errno %d, NetWareErrno %d\n",
					rVal,
					errno,
					NetWareErrno
					);
				break;
			}

		goto END_ERR;
		}

	/*------------------------------------------------------------------------
	**	Setup GetconectionsUsingFile() request buffer.
	**
	**	DirectoryID is the same thing as a directoryNumber.
	*/
	conReq.unionType 				=	ENVSERV_CONN_TYPE_386;
	conReq.req386.forkType		= 	0;				  	/* 0 = MAC data fork, or DOS; 1 = MAC resource Fork */
	conReq.req386.volume 		=	(BYTE)volumeNumber;
	conReq.req386.directoryID	=	NSDirBase;
	conReq.req386.nextRecord	=	0;					/* 0 = findFirst */

	/*------------------------------------------------------------------------
	**	Setup GetconectionsUsingFile() reply buffer.
	*/
	conRep = malloc(ENVSERV_BUFFER1_SIZE);
	if(conRep == NULL)
		{
		fprintf(stderr, "ERROR: Cannot allocat %d bytes for reply buffer.\n", ENVSERV_BUFFER1_SIZE);
		goto END_ERR;
		}
	conRep->unionType	=	ENVSERV_CONN_TYPE_386;

	/*------------------------------------------------------------------------
	**	Get all logical connections using a file.
	*/
	rVal=GetConnectionsUsingFile(
		/*	I-	requestSize	*/	sizeof(conReq), 
		/*	I-	request		*/	&conReq,
		/*	IO	buffer		*/	conRep,
		/*	I-	buffersize	*/	ENVSERV_BUFFER1_SIZE
		);
	if(rVal != ESUCCESS)
		{
		fprintf(stderr, "ERROR: GetConnectionsUsingFile() indicates error: %d  errno: %d  NetWareErrno: %d",
			rVal,
			errno,
			NetWareErrno
			);
		goto END_ERR;
		}

	/*------------------------------------------------------------------------
	**	Print results.
	*/
	printf("File................: %s\n",	fullPath);
	printf("Use Count...........: %hu\n", 	conRep->rep386.useCount);
	printf("Open Count..........: %hu\n", 	conRep->rep386.openCount);
	printf("Open for read count.: %hu\n", 	conRep->rep386.openForReadCount);
	printf("Open for write count: %hu\n", 	conRep->rep386.openForWriteCount);
	printf("Deny read count.....: %hu\n", 	conRep->rep386.denyReadCount);
	printf("Deny write count....: %hu\n", 	conRep->rep386.denyWriteCount);
	printf("File locked.........: %s\n", 	conRep->rep386.locked ? "YES" : "NO");
	printf("Fork count..........: %hu\n", 	(WORD)conRep->rep386.forkCount);
	printf("NumberOfRecords.....: %hu\n", 	(WORD)conRep->rep386.numberOfRecords);

	printf("\n");
	printf("Conn     Task   LockType   AccessFlags   LockStatus   ObjectName\n");
	printf("-----------------------------------------------------------------------------\n");
	conRepRec=(CONN_USING_FILE_RECORD_386 *)(((BYTE *)conRep) + sizeof(CONN_USING_FILE_REPLY));
	for(nCnt=1; nCnt<=conRep->rep386.numberOfRecords; ++nCnt)
		{
		rVal=GetConnectionInformation(
			/*	I-	connectionNumber	*/ IntSwap(conRepRec->connectionNumber),
			/*	-O	objectName			*/	objectName,
			/*	-O	objectType			*/	&objectType,
			/*	-O	objectID				*/	&objectID,
			/*	-O	loginTime			*/	loginTime
			);
		if(rVal != ESUCCESS) strcpy(objectName, "?");

		printf("%-4hu     %-4hu   0x%02hX       0x%02hX          0x%02hX         %s\n",
			IntSwap(conRepRec->connectionNumber),
			IntSwap(conRepRec->taskNumber),
			(WORD)conRepRec->lockType,
			(WORD)conRepRec->accessFlags,
			(WORD)conRepRec->lockStatus,
			objectName
			);

		++conRepRec;
		}

END_ERR:

	if(conRep != NULL)
		free(conRep);

	exit(0);
	}
